@@ -2,6 +2,7 @@ use crate::rule::{rule_by_name, Rule, Violation};
 use regex::Regex;
 
 lazy_static! {
+    static ref SUBJECT_STARTS_WITH_PREFIX: Regex = Regex::new(r"^([\w\(\)/!]+:)\s.*").unwrap();
     static ref SUBJECT_STARTS_WITH_EMOJI: Regex = Regex::new(r"^\p{Emoji}").unwrap();
     static ref SUBJECT_WITH_TICKET: Regex = Regex::new(r"[A-Z]+-\d+").unwrap();
     // Match all GitHub and GitLab keywords
@@ -57,6 +58,34 @@ lazy_static! {
         "tested",
         "testing",
     ];
+    static ref BUILD_TAGS: Vec<&'static str> = vec![
+        // General
+        "[ci skip]",
+        "[skip ci]",
+        "[no ci]",
+        // AppVeyor
+        "[skip appveyor]",
+        // Azure
+        "[azurepipelines skip]",
+        "[skip azurepipelines]",
+        "[azpipelines skip]",
+        "[skip azpipelines]",
+        "[azp skip]",
+        "[skip azp]",
+        "***NO_CI***",
+        // GitHub Actions
+        "[actions skip]",
+        "[skip actions]",
+        // Travis
+        "[travis skip]",
+        "[skip travis]",
+        "[travis ci skip]",
+        "[skip travis ci]",
+        "[travis-ci skip]",
+        "[skip travis-ci]",
+        "[travisci skip]",
+        "[skip travisci]",
+    ];
 }
 
 #[derive(Debug)]
@@ -124,6 +153,8 @@ impl Commit {
         self.validate_subject_capitalization();
         self.validate_subject_punctuation();
         self.validate_subject_ticket_numbers();
+        self.validate_subject_prefix();
+        self.validate_subject_build_tags();
         self.validate_subject_cliches();
         self.validate_message_second_line_empty();
         self.validate_message_presence();
@@ -323,29 +354,70 @@ impl Commit {
         }
     }
 
+    fn validate_subject_prefix(&mut self) {
+        if self.rule_ignored(Rule::SubjectPrefix) {
+            return;
+        }
+
+        let subject = &self.subject.to_string();
+        if let Some(captures) = SUBJECT_STARTS_WITH_PREFIX.captures(subject) {
+            // Get first match from captures, the prefix
+            match captures.get(1) {
+                Some(capture) => self.add_violation(
+                    Rule::SubjectPrefix,
+                    format!(
+                        "Remove the prefix from the commit subject: \"{}\"",
+                        capture.as_str()
+                    ),
+                ),
+                None => error!("SubjectPrefix: Unable to fetch prefix capture from subject."),
+            }
+        }
+    }
+
+    fn validate_subject_build_tags(&mut self) {
+        if self.rule_ignored(Rule::SubjectBuildTag) {
+            return;
+        }
+
+        let subject = &self.subject.to_string();
+        for tag in BUILD_TAGS.iter() {
+            if subject.contains(tag) {
+                self.add_violation(
+                    Rule::SubjectBuildTag,
+                    format!("Move the build tag `{}` to the message body.", tag),
+                )
+            }
+        }
+    }
+
     fn validate_subject_cliches(&mut self) {
         if self.rule_ignored(Rule::SubjectCliche) {
             return;
         }
 
-        let subject = &self.subject;
-        let wip_commit =
-            subject.to_lowercase().starts_with("wip ") || subject.to_lowercase() == *"wip";
+        let subject = &self.subject.to_lowercase();
+        let wip_commit = subject.starts_with("wip ") || subject == &"wip".to_string();
         if wip_commit {
             self.add_violation(
                 Rule::SubjectCliche,
                 "Reword the subject to describe the change in more detail.".to_string(),
             )
-        } else if subject == &"Fix test".to_string() {
+        } else if subject == &"fix test".to_string() {
             self.add_violation(
                 Rule::SubjectCliche,
                 "Reword the subject to explain which test was fixed.".to_string(),
             )
-        } else if subject == &"Fix bug".to_string() {
+        } else if subject == &"fix bug".to_string() {
             self.add_violation(
                 Rule::SubjectCliche,
                 "Reword the subject to explain what bug was fixed.".to_string(),
             )
+        } else if subject == &"fix".to_string() {
+            self.add_violation(
+                Rule::SubjectCliche,
+                "Reword the subject to explain what was fixed.".to_string(),
+            )
         }
     }
 
@@ -453,7 +525,7 @@ enum CodeBlockStyle {
 
 #[cfg(test)]
 mod tests {
-    use super::{Commit, Rule, Violation, MOOD_WORDS};
+    use super::{Commit, Rule, Violation, BUILD_TAGS, MOOD_WORDS};
 
     fn commit_with_sha(sha: Option<String>, subject: String, message: String) -> Commit {
         Commit::new(sha, subject, message)
@@ -500,12 +572,12 @@ mod tests {
         }
     }
 
-    fn assert_commit_subject_as_invalid(subject: &str, rule: &Rule) {
-        let commit = validated_commit(subject.to_string(), "".to_string());
+    fn assert_commit_subject_as_invalid<S: AsRef<str>>(subject: S, rule: &Rule) {
+        let commit = validated_commit(subject.as_ref().to_string(), "".to_string());
         assert_commit_invalid_for(commit, rule);
     }
 
-    fn assert_commit_subjects_as_invalid(subjects: Vec<&str>, rule: &Rule) {
+    fn assert_commit_subjects_as_invalid<S: AsRef<str>>(subjects: Vec<S>, rule: &Rule) {
         for subject in subjects {
             assert_commit_subject_as_invalid(subject, rule)
         }
@@ -741,6 +813,60 @@ mod tests {
         assert_commit_valid_for(ignore_issue_number, &Rule::SubjectTicketNumber);
     }
 
+    #[test]
+    fn test_validate_subject_prefix() {
+        let subjects = vec!["This is a commit without prefix"];
+        assert_commit_subjects_as_valid(subjects, &Rule::SubjectPrefix);
+
+        let invalid_subjects = vec![
+            "fix: bug",
+            "fix!: bug",
+            "Fix: bug",
+            "Fix!: bug",
+            "fix(scope): bug",
+            "fix(scope)!: bug",
+            "Fix(scope123)!: bug",
+            "fix(scope/scope): bug",
+            "fix(scope/scope)!: bug",
+        ];
+        assert_commit_subjects_as_invalid(invalid_subjects, &Rule::SubjectPrefix);
+
+        let commit = validated_commit("fix: bug".to_string(), "".to_string());
+        let violation = commit
+            .violations
+            .iter()
+            .find(|&v| v.rule == Rule::SubjectPrefix)
+            .unwrap_or_else(|| panic!("Could not find violation"));
+        assert_eq!(
+            violation.message,
+            "Remove the prefix from the commit subject: \"fix:\""
+        );
+
+        let ignore_commit = validated_commit(
+            "fix: bug".to_string(),
+            "lintje:disable SubjectPrefix".to_string(),
+        );
+        assert_commit_valid_for(ignore_commit, &Rule::SubjectPrefix);
+    }
+
+    #[test]
+    fn test_validate_subject_build_tags() {
+        let subjects = vec!["Add exception for no ci build tag"];
+        assert_commit_subjects_as_valid(subjects, &Rule::SubjectBuildTag);
+
+        let mut invalid_subjects = vec![];
+        for tag in BUILD_TAGS.iter() {
+            invalid_subjects.push(format!("Update README {}", tag))
+        }
+        assert_commit_subjects_as_invalid(invalid_subjects, &Rule::SubjectBuildTag);
+
+        let ignore_commit = validated_commit(
+            "Update README [ci skip]".to_string(),
+            "lintje:disable SubjectBuildTag".to_string(),
+        );
+        assert_commit_valid_for(ignore_commit, &Rule::SubjectBuildTag);
+    }
+
     #[test]
     fn test_validate_subject_cliches() {
         let subjects = vec![
@@ -757,7 +883,12 @@ mod tests {
             "WIP",
             "wip",
             "Fix test",
+            "fix test",
             "Fix bug",
+            "fix bug",
+            "Fix",
+            "fix",
+            "FIX",
         ];
         assert_commit_subjects_as_invalid(invalid_subjects, &Rule::SubjectCliche);
 
